@@ -10,10 +10,16 @@ The service coordinates the conversion workflow using dependency injection.
 from __future__ import annotations
 
 import io
+import logging
 import time
 from collections.abc import Sequence
 from dataclasses import dataclass, field
 from datetime import datetime
+from zipfile import ZipFile
+
+# Constants for metadata extraction
+DATETIME_STRING_LENGTH = 14  # YYYYMMDDHHmmSS format
+PDF_METADATA_PREFIX_LENGTH = 2  # Remove "D:" prefix from PDF metadata
 
 from ..domain import (
     ConversionFailed,
@@ -21,6 +27,7 @@ from ..domain import (
     ExtractionResult,
     Extractor,
     Markdown,
+    NormalizedDoc,
     Policy,
     QualityScorer,
     RecoverableError,
@@ -57,10 +64,12 @@ def _extract_document_metadata(doc: Document) -> dict[str, str]:
                     created_date_str = pdf_metadata["creationDate"]
                     # Parse the PDF date format
                     if created_date_str.startswith("D:"):
-                        created_date_str = created_date_str[2:]
+                        created_date_str = created_date_str[PDF_METADATA_PREFIX_LENGTH:]
                     # Take first 14 chars (YYYYMMDDHHmmSS)
-                    if len(created_date_str) >= 14:
-                        dt = datetime.strptime(created_date_str[:14], "%Y%m%d%H%M%S")
+                    if len(created_date_str) >= DATETIME_STRING_LENGTH:
+                        dt = datetime.strptime(
+                            created_date_str[:DATETIME_STRING_LENGTH], "%Y%m%d%H%M%S"
+                        )
                         metadata["created_utc"] = dt.isoformat()
 
                 pdf_doc.close()
@@ -71,14 +80,18 @@ def _extract_document_metadata(doc: Document) -> dict[str, str]:
         elif doc.mime == "application/vnd.openxmlformats-officedocument.wordprocessingml.document":
             # Try to extract DOCX metadata
             try:
-                import xml.etree.ElementTree as ET
-                from zipfile import ZipFile
-
                 with ZipFile(io.BytesIO(doc.bytes)) as docx_zip:
                     # Read core properties
                     if "docProps/core.xml" in docx_zip.namelist():
                         core_xml = docx_zip.read("docProps/core.xml")
-                        root = ET.fromstring(core_xml)
+                        # Use defusedxml for safer XML parsing
+                        try:
+                            import defusedxml.ElementTree as ET
+
+                            root = ET.fromstring(core_xml)
+                        except ImportError:
+                            # Fallback to standard library if defusedxml not available
+                            root = ET.fromstring(core_xml)
 
                         # Look for created date
                         # Namespace for Dublin Core
@@ -103,13 +116,17 @@ def _extract_document_metadata(doc: Document) -> dict[str, str]:
         ]:
             # Try to extract Excel metadata
             try:
-                import xml.etree.ElementTree as ET
-                from zipfile import ZipFile
-
                 with ZipFile(io.BytesIO(doc.bytes)) as xlsx_zip:
                     if "docProps/core.xml" in xlsx_zip.namelist():
                         core_xml = xlsx_zip.read("docProps/core.xml")
-                        root = ET.fromstring(core_xml)
+                        # Use defusedxml for safer XML parsing
+                        try:
+                            import defusedxml.ElementTree as ET
+
+                            root = ET.fromstring(core_xml)
+                        except ImportError:
+                            # Fallback to standard library if defusedxml not available
+                            root = ET.fromstring(core_xml)
 
                         ns = {
                             "dcterms": "http://purl.org/dc/terms/",
@@ -209,12 +226,10 @@ class ConverterService:
                     try:
                         max_images = 5  # default
                         if self.llm_config and hasattr(self.llm_config, "max_images_per_document"):
-                            max_images = self.llm_config.max_images_per_document  # type: ignore
+                            max_images = self.llm_config.max_images_per_document
                         ndoc = self._enhance_images_with_descriptions(ndoc, max_images)
                     except Exception as e:
                         # Don't fail entire conversion if LLM fails
-                        import logging
-
                         logging.error(f"LLM image description failed, continuing without: {e}")
 
                 # Apply table profilers if configured
@@ -396,7 +411,7 @@ class ConverterService:
 
     def get_supported_formats(self) -> Sequence[str]:
         """Get list of all supported MIME types."""
-        all_extractors = []
+        all_extractors: list[Extractor] = []
         for precision in [1, 2, 3, 4]:
             extractors = self.policy.choose_extractors(
                 mime="application/pdf",  # dummy
@@ -408,7 +423,7 @@ class ConverterService:
 
         # Collect unique MIME types
         formats: set[str] = set()
-        for extractor in all_extractors:
+        for _extractor in all_extractors:
             # This is a simplification - in practice, we'd query each extractor
             # for its supported formats
             pass
@@ -430,8 +445,6 @@ class ConverterService:
         Returns:
             Enhanced document with image descriptions
         """
-        import logging
-
         from ..app.image_context import ImageContextTracker
         from ..domain import Block, BlockType, NormalizedDoc
 
@@ -442,7 +455,7 @@ class ConverterService:
 
         tracker = ImageContextTracker(context_lines=100)
         if self.llm_config and hasattr(self.llm_config, "context_lines"):
-            tracker = ImageContextTracker(context_lines=self.llm_config.context_lines)  # type: ignore
+            tracker = ImageContextTracker(context_lines=self.llm_config.context_lines)
 
         enhanced_blocks = []
         images_described = 0
@@ -453,9 +466,9 @@ class ConverterService:
                 tracker.add_text(block.content)
                 enhanced_blocks.append(block)
 
-            # Process images (up to max limit)
+            # Process images (up to max limit, or unlimited if -1)
             elif block.type == BlockType.IMAGE:
-                if block.image_data and images_described < max_images:
+                if block.image_data and (max_images == -1 or images_described < max_images):
                     try:
                         context = tracker.get_context()
                         description = self.image_describer.describe_image(  # type: ignore
@@ -465,7 +478,7 @@ class ConverterService:
                         # Create new block with description
                         enhanced_block = Block(
                             type=BlockType.IMAGE,
-                            content=f"{block.content}\n\n**Image Description**: {description}",
+                            content=description,
                             metadata={
                                 **block.metadata,
                                 "llm_described": True,
@@ -477,8 +490,9 @@ class ConverterService:
                         enhanced_blocks.append(enhanced_block)
                         images_described += 1
 
+                        max_images_str = "unlimited" if max_images == -1 else str(max_images)
                         logger.info(
-                            f"Described image {images_described}/{max_images} "
+                            f"Described image {images_described}/{max_images_str} "
                             f"on page {block.page_number}"
                         )
 
