@@ -10,6 +10,7 @@ import tempfile
 import time
 import warnings
 from pathlib import Path
+from typing import Any
 
 from ...domain import (
     Block,
@@ -112,14 +113,19 @@ class DoclingExtractor:
                     warnings.filterwarnings(
                         "ignore", message="invalid value encountered in scalar divide"
                     )
-                    result = self._converter.convert(temp_path)
+                    result: Any = self._converter.convert(temp_path)
                     markdown_content = result.document.export_to_markdown()
 
                 # Extract raw images from document (for LLM description)
                 image_data_by_page = self._extract_images_from_file(temp_path)
 
+                # Extract image page mapping from Docling's structured API
+                image_page_mapping = self._extract_image_page_mapping(result.document)
+
                 # Parse markdown into blocks
-                blocks = self._parse_markdown_to_blocks(markdown_content, image_data_by_page)
+                blocks = self._parse_markdown_to_blocks(
+                    markdown_content, image_data_by_page, image_page_mapping
+                )
 
                 # Extract metadata from Docling result
                 page_count = None
@@ -192,13 +198,18 @@ class DoclingExtractor:
             )
 
     def _parse_markdown_to_blocks(
-        self, markdown: str, image_data_by_page: dict[int, list[bytes]] | None = None
+        self,
+        markdown: str,
+        image_data_by_page: dict[int, list[bytes]] | None = None,
+        image_page_mapping: list[int] | None = None,
     ) -> list[Block]:
         """
         Parse Docling markdown into structured blocks.
 
         Args:
             markdown: Markdown from Docling
+            image_data_by_page: Dictionary mapping page numbers to image bytes
+            image_page_mapping: List mapping image index to page number
 
         Returns:
             List of Block objects
@@ -210,6 +221,7 @@ class DoclingExtractor:
         in_table = False
         table_lines: list[str] = []
         image_index_by_page: dict[int, int] = {}  # Track image index per page
+        global_image_index = 0  # Track global image index for page mapping
 
         for line in lines:
             # Detect headers
@@ -296,24 +308,30 @@ class DoclingExtractor:
                         )
                     current_text = []
 
+                # Determine page number for this image
+                image_page = current_page
+                if image_page_mapping and global_image_index < len(image_page_mapping):
+                    image_page = image_page_mapping[global_image_index]
+
                 # Get image data if available
                 img_data = None
                 if image_data_by_page:
-                    page_images = image_data_by_page.get(current_page, [])
-                    img_idx = image_index_by_page.get(current_page, 0)
+                    page_images = image_data_by_page.get(image_page, [])
+                    img_idx = image_index_by_page.get(image_page, 0)
 
                     if img_idx < len(page_images):
                         img_data = page_images[img_idx]
-                    image_index_by_page[current_page] = img_idx + 1
+                    image_index_by_page[image_page] = img_idx + 1
 
                 blocks.append(
                     Block(
                         type=BlockType.IMAGE,
-                        content=line.strip(),
-                        page_number=current_page,
+                        content="",
+                        page_number=image_page,
                         image_data=img_data,
                     )
                 )
+                global_image_index += 1
                 continue
 
             # Accumulate regular text
@@ -431,3 +449,36 @@ class DoclingExtractor:
             logger.warning(f"Image extraction failed: {e}")
 
         return images
+
+    def _extract_image_page_mapping(self, document: Any) -> list[int]:
+        """
+        Extract page numbers for each image from Docling's document structure.
+
+        Args:
+            document: Docling Document object
+
+        Returns:
+            List mapping image index to page number (1-indexed)
+        """
+        import logging
+
+        logger = logging.getLogger(__name__)
+        page_mapping: list[int] = []
+
+        try:
+            # Iterate through document items to find pictures
+            for item, _level in document.iterate_items(with_groups=False):
+                # Check if this is a picture item
+                if hasattr(item, "label") and "picture" in str(item.label).lower():
+                    # Extract page number from provenance
+                    if hasattr(item, "prov") and item.prov:
+                        page_no = item.prov[0].page_no
+                        page_mapping.append(page_no)
+                        logger.debug(f"Image at index {len(page_mapping) - 1} is on page {page_no}")
+
+            logger.info(f"Extracted page mapping for {len(page_mapping)} image(s)")
+
+        except Exception as e:
+            logger.warning(f"Failed to extract image page mapping: {e}")
+
+        return page_mapping
